############################################################################################################################################################################
# ONE STATION METABOLISM CODE for Methods in Stream Ecology, 3rd edition (Hall & Hotchkiss)

#This code provides all of the functions necessary to estimate single station open-channel stream metaboism from diel O2.  
#The methods to estimate metabolism are via exact solution or "bookkeeping".  
#We also include 2 data files (S4 & S5)

#Bob Hall 7 Oct 2015.
#updated annotation, Erin Hotckhiss 2016-10-31
############################################################################################################################################################################



###########################################
# [1] DATA IMPORT AND MANAGEMENT
###########################################	


## Call and name the O2 logger data. Use your own path specific to your computer.
spring<-read.csv("/Users/bobhall/Rivers/spring14.csv")  		##data from Spring Creek WY, across from Bob's house
french<-read.csv("/Users/bobhall/O2 modeling erin/french1.csv")	##data from French Creek WY, Hotchkiss & Hall (2015) Ecology

## for this code to work you need your data looking like the example datasets below:
	## temp = water temperature in Celcius
	## oxy = dissolved oxygen in mg/L
head(spring)  	##time is in seconds since 1970 and in UTC
head(french)	##time is date and time in separate columns. Follow the date and time format exactly (i.e. seconds on the time as hh:mm:ss; mm/dd/yy for date)

## TO USE DATES & TIMES IN R: First make make date and time a chron object. Load package chron from the R website and look online or via "help(chron)" for how it works.  
## How you make a chron object depends on your data. PME MiniDOTS give data in seconds since 1970 whichs makes for a really easy translation into a chron object.  Hydrolabs are a bit tougher.  Below are two ways of doing it.
## If not already installed, first enter command "install.packages("chron")"

## Load chron package to work with dates/times in R
library(chron)

## PME Minidot: make time correction (versus UTC) and chron object
## Minidots report time in UTC and give option to adjust time with miniDOT Plot software <or> can do it here in R with original CAT file (we prefer to do it in R).

spring$dtime<-chron(spring$time/86400)-(6/24)	##the subtraction of 6/24 (0.25 d) is because local time in WY is 6 h behind UTC in summer.  

## The below makes a chron object for dates and times where each is in its own column
french$dtime<-chron(dates=as.character(french $date), times=as.character(french $time))

## ONLY IF NEEDED: if date and time are in the same column; split the string (underthe header 'datetime'). Make sure your datetime is to the second!  
## Format is mm/dd/yy hh:mm:ss  The function strsplit will split at the space bewteen date and time. 
## Change "YOURDATA" to whatever you named your data file (as we did for "spring" or "french")
dtparts<-t(as.data.frame(strsplit(as.character(YOURDATA$datetime),' ')))
row.names(dtparts)=NULL
## make chron object
YOURDATA$dtime<-chron(dates=dtparts[,1], times=dtparts[,2])

## Check new chron object dtime and O2 data
## here is where you can target and remove outliers if you want to check before running models 
plot(spring$dtime, spring$oxy)
plot(french$dtime, french$oxy)

####### END data important and management ####### 

####### 


###########################################
# [2] LOAD O2 SATURATION FUNCTION
###########################################	

## LOAD function to calculate oxygen saturation given water temperature and barometric pressure.

## oxygen saturation. From García and Gordonn (1992).
	## calculates mL gas / dm^3 water; the below equation converts to mg/L.    
	## BP in mm Hg. 
	## 1.42905 is 'correct' and accounts for the fact that O2 is a non-ideal gas.  
	## u is the vapor pressure of water

osat<- function(temp, bp) {
	
	tstd<-log((298.15-temp) / (273.15 + temp))
	
	a0<-2.00907
	a1<-3.22014
	a2<-4.0501
	a3<-4.94457
	a4<- -0.256847
	a5<- 3.88767
	
	u<-10^(8.10765-(1750.286/(235+temp)))
	
	sato<-(exp(a0 + a1*tstd + a2*tstd^2 + a3*tstd^3 + a4*tstd^4+a5*tstd^5))*((bp-u)/(760-u))*1.42905
	sato
}

####### END loading O2 SAT calc ####### 

###########################################
# [3] LOAD BAROMETRIC PRESSURE FUNCTION
###########################################	

## Function to correct barometric pressure for altitude. From Colt (2012).
## This function gives bp in mmHg for altitude given nearby measurement of standardized barometric pressure. 
	## temp is degC 
	## alt is m
	## bpst is in inches of Hg and the sort of data you will find from weather stations.  

####function returns mm of Hg
bpcalc<- function(bpst, alt) {
bpst*25.4*exp((-9.80665*0.0289644*alt)/(8.31447*(273.15+15)))
}
#end of function


####### END loading BP calc function ####### 


###########################################
# [4] LOAD GAS EXCHANGE (K) FUNCTIONS
###########################################	

# NOTE: The functions you use will depend on the methods used to estimate K (O2, propane, SF6). The temperature correction (Kcor) is embedded in the models below, but the Kcor function must be loaded into R before running the model.

# UNITS are day-1
	
## This code does the opposite of Kcor below; it estimates K600 for KO2 at a given temperature. From Wanninkhof (1992).
K600fromO2<-function (temp, KO2) {
    ((600/(1800.6 - (120.1 * temp) + (3.7818 * temp^2) - (0.047608 * temp^3)))^-0.5) * KO2
}
#end of function

## This calculates K600 from K measured in a propane addition. From Raymond et al. (2012).
K600frompropane<-function (temp, Kpropane) {
    ((600/(2864 - (154.14 * temp) + (3.791 * temp^2) - (0.0379 * temp^3)))^-0.5) * Kpropane
}
#end of function

## This calculates K600 from K measured in a SF6 addition. From Raymond et al. (2012).
K600fromSF6<-function(temp, KSF6) {
	((600/(3255.3-(217.13*temp)+(6.837*temp^2)-(0.08607*temp^3)))^-0.5)*KSF6
}
#end of function

## Gas exchange is sensitive to temperature. This code converts K600 to value at specific temp by Schmidt number scaling. From Jähne et al. (1987).
Kcor<-function (temp,K600) {
	K600/(600/(1800.6-(temp*120.1)+(3.7818*temp^2)-(0.047608*temp^3)))^-0.5
}
#end of function

####### END loading K functions ####### 



###########################################
# [5] NIGHTTIME REGRESSION to estimate K -- OPTIONAL
###########################################	
	
### nighttime regression code to estimate K. Approach as in Hornberger and Kelly (1975).
	## o2 file is your oxygen data (defined in subsetting)
	## bp is barometric pressure in mm Hg for your site, 
	## ts is the time step in MINUTES (not days as in metabolism code below)
	
nightreg<-function(o2file, bp, ts){

	temp<-o2file$temp
	oxy<-o2file$oxy

# moving average on oxy data
oxyf1<-filter(o2file$oxy,rep(1/3,3), sides=2)

# trim the ends of the oxy data
oxyf2<- oxyf1[c(-1,-length(oxyf1))]

# calculate delO/delt; convert to units of days by ts in min-1*1440
deltaO2<-((oxyf2[-1]-oxyf2[-length(oxyf2)])/ts)*1440

# Trim the first two and last one from the temp data to match the filter oxy data
temptrim<-temp[c(-2:-1,-length(temp))]

# calc the dodef
satdef<-osat(temptrim,bp)-oxyf2[-1]

# fit linear model and plot using linear model fitting (lm) and abline functions in R stats package
nreg<-lm(deltaO2~satdef)
plot(satdef,deltaO2)
abline(nreg)

# use coef function in R stats package to get lm coefficients
coeff<-coef(nreg)

# output gives lm coeff and K600 (converted from nighttime regression estimate of KO2)
out<-list(coeff, K600fromO2(mean(temp), coeff[2]))
out

}
#end of function

# NOTE: this approach works better for some sites/dates than others; always check that your model fit is good and that your K600 estimate makes sense!

#Call as:  The first argument in the function defines when to pull data.  In this case on 10/27/204 (for spring creek) between 18:05 and 23:00
	
	nightreg(spring[spring$dtime>=as.numeric(chron(dates="10/27/14", times="18:05:00")) & spring$dtime<=as.numeric(chron(dates="10/27/14", times="23:00:00")), ], bp=595, ts=10)
	
	nightreg(french[french$dtime>=as.numeric(chron(dates="09/21/12", times="19:40:00")) & french$dtime<=as.numeric(chron(dates="09/21/12", times="23:00:00")), ], bp=523, ts=5)
	
	nightreg(french[french$dtime>=as.numeric(chron(dates="09/22/12", times="19:40:00")) & french$dtime<=as.numeric(chron(dates="09/22/12", times="23:00:00")), ], bp=523, ts=5)

####### END nighttime regression calculation of K ####### 


###########################################
# [6] DIRECT CALC MODEL - METABCALC; solves for GPP, ER
###########################################

###function to calculate metabolism as a direct calculation (Eq 34.14 in chapter) 

metabcalc<-function(o2file, K600, z, bp, ts, sunrise, sunset) {
	
	temp<-o2file$temp
	oxy<-o2file$oxy
	oxysat<-osat(o2file$temp,bp)
	dtime<-o2file$dtime
	
	###calculate net metabolism at each time step.  Eq. 34.14 
	metab<-((diff(oxy)/ts) - Kcor(temp[-1], K600)*(oxysat[-1]-oxy[-1]))*z

##Make a dataframe with dates, net metabolism and times (units of time is day)
out<-data.frame(dtime=dtime[-1], metab=metab, time=as.numeric(dtime[-1]-trunc(as.numeric(dtime[-1]))  )   )

##define sunrise and sunset as fraction of day
sunriset<- as.numeric (chron(date="01/01/1970", time=sunrise))
sunsett<- as.numeric (chron(date="01/01/1970", time=sunset))

#calculate ER and mean of net metab before sunrise and after sunset
ER<- mean (c(out$metab[out$time<sunriset], out$metab[out$time>sunsett]))

#subset out to only include daytime
dayout<- out[out$time > sunriset & out$time < sunsett, ]

#integrate net metabolism in the daytime to estimate GPP  ###SOMETHING WRONG HERE
ydiff<- dayout$metab[-1]-ER+ dayout$metab[-length(dayout$metab)]-ER
  GPP<-sum(diff(dayout$time)*ydiff/2)

	
	plot(dtime[-1],metab)
	return(list(GPP=GPP, ER=ER))
	
}

####### END LOADING metabcalc function ####### 


###########################################
# [7] CALL METABCALC function
###########################################

metabcalc(o2file=spring[ spring $dtime>=as.numeric(chron(dates="10/27/14", times="22:00:00")) & spring $dtime<=as.numeric(chron(dates="10/29/14", times="06:00:00")), ], K600=25.7, z=0.18, bp=595, ts=0.006944, sunrise="06:00:00", sunset="18:00:00")

####### END example CALL of metabcalc function ####### 


